"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
require("source-map-support/register");
const yaml = __importStar(require("js-yaml"));
/**
 * Get the service name as at stack creation time
 * @param service
 */
function getServiceName(service) {
    if (service.Spec.Labels) {
        for (const label in service.Spec.Labels) {
            if (label === 'com.docker.stack.namespace') {
                return service.Spec.Name.replace(service.Spec.Labels[label] + '_', '');
            }
        }
    }
    return service.Spec.Name;
}
/**
 * Remove Docker-internal labels added by Docker CLI
 * @param labels
 */
function stripDockerLabels(labels) {
    const result = {};
    if (labels) {
        for (const key in labels) {
            if (!(key === 'com.docker.stack.image') && !(key === 'com.docker.stack.namespace')) {
                result[key] = labels[key];
            }
        }
    }
    return result;
}
function parseMounts(service) {
    const mounts = [];
    if (service.Spec.TaskTemplate.ContainerSpec.Mounts) {
        const dockerMounts = service.Spec.TaskTemplate.ContainerSpec.Mounts;
        for (const dockerMount of dockerMounts) {
            let stackPrefix = '';
            if (dockerMount.VolumeOptions && dockerMount.VolumeOptions.Labels) {
                if (dockerMount.VolumeOptions.Labels['com.docker.stack.namespace']) {
                    stackPrefix = dockerMount.VolumeOptions.Labels['com.docker.stack.namespace'] + '_';
                }
            }
            mounts.push({
                type: dockerMount.Type,
                source: dockerMount.Source.replace(stackPrefix, ''),
                target: dockerMount.Target,
            });
        }
    }
    return mounts;
}
function getNetworkName(network) {
    if (network.Labels) {
        for (const label in network.Labels) {
            if (label === 'com.docker.stack.namespace') {
                return network.Name.replace(network.Labels[label] + '_', '');
            }
        }
    }
    // Network was not created as part of a stack
    return network.Name;
}
/**
 * Returns the networks from the NetworkMap according to the given service
 * @param {DockerModels.Service} service
 * @param {NetworkMap} networkMap
 * @returns {string[]}
 */
function parseNetworks(service, networkMap) {
    const result = [];
    const networks = service.Spec.TaskTemplate.Networks;
    if (networks) {
        for (const network of networks) {
            if (network.Target) {
                if (networkMap[network.Target]) {
                    result.push(networkMap[network.Target]);
                }
                else {
                    throw new Error('Network ID not found in NetworkMap: ' + network.Target);
                }
            }
        }
    }
    return result;
}
/**
 * Create intermediate structure to help with Stack networks
 *
 * @param {NetworkMap} networkMap
 * @returns {{[name: string]: ComposeModels.StackNetwork}}
 */
function createStackNetworkMap(networkMap) {
    const result = {};
    for (const networkId in networkMap) {
        const dockerNetwork = networkMap[networkId];
        if (!dockerNetwork) {
            throw new Error('undefined network in NetworkMap');
        }
        if (!dockerNetwork.Driver) {
            throw new Error('Network in NetworkMap does not have a driver: ' + networkId);
        }
        result[getNetworkName(networkMap[networkId])] = {
            driver: dockerNetwork.Driver,
        };
        if (dockerNetwork.Labels && !(dockerNetwork.Labels["com.docker.stack.namespace"])) {
            result[getNetworkName(networkMap[networkId])].external = true;
        }
    }
    return result;
}
function parsePorts(service) {
    const result = [];
    // NOTE: assumes the attributes will be defined
    if (service.Spec.EndpointSpec && service.Spec.EndpointSpec.Ports) {
        for (const port of service.Spec.EndpointSpec.Ports) {
            result.push({
                published: port.PublishedPort,
                target: port.TargetPort,
                protocol: port.Protocol,
                mode: port.PublishMode,
            });
        }
    }
    return result;
}
function getConfigNames(service) {
    const result = [];
    if (service.Spec.TaskTemplate.ContainerSpec.Configs) {
        for (const config of service.Spec.TaskTemplate.ContainerSpec.Configs) {
            // NOTE: Assumes ConfigName will be defined
            // TODO: Remove stack prefix
            result.push(config.ConfigName);
        }
    }
    return result;
}
function parseService(service, networkMap) {
    // Image name
    const result = {
        image: service.Spec.TaskTemplate.ContainerSpec.Image.split('@')[0],
    };
    // Ports
    const ports = parsePorts(service);
    if (ports.length !== 0) {
        result.ports = ports;
    }
    // Networks
    const networks = parseNetworks(service, networkMap);
    if (networks.length !== 0) {
        for (const net of networks) {
            result.networks = [];
            result.networks.push(getNetworkName(net));
        }
    }
    // Configs
    // TODO:
    const deployBlob = {};
    // Mode
    if (service.Spec.Mode && service.Spec.Mode.Replicated) {
        deployBlob.mode = "replicated";
        deployBlob.replicas = service.Spec.Mode.Replicated.Replicas;
    }
    if (service.Spec.Mode && service.Spec.Mode.Global) {
        deployBlob.mode = "global";
    }
    // Mounts
    const mounts = parseMounts(service);
    if (mounts.length !== 0) {
        result.volumes = mounts;
    }
    // Labels
    const labelsService = stripDockerLabels(service.Spec.Labels);
    const labelsContainer = stripDockerLabels(service.Spec.TaskTemplate.ContainerSpec.Labels);
    // Constraints
    let constraints = [];
    if (service.Spec.TaskTemplate.Placement && service.Spec.TaskTemplate.Placement.Constraints) {
        constraints = service.Spec.TaskTemplate.Placement.Constraints;
    }
    // Labels attached to service
    if (Object.keys(labelsService).length !== 0) {
        deployBlob.labels = labelsService;
    }
    // Labels attached to container
    if (Object.keys(labelsContainer).length !== 0) {
        result.labels = labelsContainer;
    }
    if (constraints.length !== 0) {
        deployBlob.placement = { constraints };
    }
    // Attach deploy section if required
    if (Object.keys(labelsService).length + constraints.length > 0 || Object.keys(deployBlob).length > 0) {
        result.deploy = deployBlob;
    }
    if (service.Spec.TaskTemplate.ContainerSpec.Env) {
        result.environment = service.Spec.TaskTemplate.ContainerSpec.Env;
    }
    return result;
}
/**
 * Generate a docker-compose string from a list of JSON service documents
 *
 * @param services Array of Services to generate a compose file from
 * @param networks Map of network ID -> network name
 */
function compose(services, networkMap, volumeMap) {
    const stackNetworkMap = createStackNetworkMap(networkMap);
    const servicesBlob = {};
    for (const dockerService of services) {
        const composeService = parseService(dockerService, networkMap);
        servicesBlob[getServiceName(dockerService)] = composeService;
    }
    // TODO(egeldenhuys): Move networks and volumes to own function?
    const networksBlob = {};
    const volumesBlob = {};
    for (const serviceName in servicesBlob) {
        const composeService = servicesBlob[serviceName];
        if (composeService.networks) {
            for (const networkName of composeService.networks) {
                if (!stackNetworkMap[networkName]) {
                    throw new Error('undefined network in StackNetworkMap: ' + networkName);
                }
                networksBlob[networkName] = stackNetworkMap[networkName];
            }
        }
        if (composeService.volumes) {
            for (const mount of composeService.volumes) {
                if (mount.type === "volume") {
                    if (!volumeMap[mount.source]) {
                        throw new Error('undefined volume in VolumeMap. Volume name: ' + mount.source);
                    }
                    if (volumeMap[mount.source].Labels &&
                        !(volumeMap[mount.source].Labels['com.docker.stack.namespace'])) {
                        volumesBlob[mount.source] = {
                            external: true,
                        };
                    }
                    else {
                        volumesBlob[mount.source] = {
                            driver: volumeMap[mount.source].Driver,
                        };
                    }
                }
            }
        }
    }
    const stackFile = {
        version: '3.3',
        services: servicesBlob,
    };
    if (Object.keys(networksBlob).length !== 0) {
        stackFile.networks = networksBlob;
    }
    if (Object.keys(volumesBlob).length !== 0) {
        stackFile.volumes = volumesBlob;
    }
    return yaml.safeDump(stackFile);
}
exports.compose = compose;
/**
 * Creates a map of volume name => volume and normalizes the volume name
 * @param volumes
 */
function createVolumeMap(volumes) {
    const result = {};
    for (const vol of volumes) {
        if (vol.Labels && vol.Labels["com.docker.stack.namespace"]) {
            vol.Name = vol.Name.replace(vol.Labels["com.docker.stack.namespace"] + "_", "");
        }
        result[vol.Name] = vol;
    }
    return result;
}
exports.createVolumeMap = createVolumeMap;
/**
 * Create a network map to allow generating a docker-compose file
 * @param networks Array of Network objects from Docker
 */
function createNetworkMap(networks) {
    const result = {};
    for (const net of networks) {
        result[net.Id] = net;
    }
    return result;
}
exports.createNetworkMap = createNetworkMap;
/**
 * Return a list of network IDs that require inspection
 *
 * This is required to create a NetworkMap
 * @param services
 */
function getNetworkIds(services) {
    const result = [];
    for (const service of services) {
        if (service.Spec.TaskTemplate.Networks) {
            for (const net of service.Spec.TaskTemplate.Networks) {
                result.push(net.Target);
            }
        }
    }
    return result;
}
exports.getNetworkIds = getNetworkIds;
exports.default = compose;
//# sourceMappingURL=index.js.map